// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "BlueNoiseSampler.h"
#include <fstream>

BlueNoiseSampler::BlueNoiseSampler(uint32_t textureResolution,uint32_t textureCount):
randomGenerator(std::random_device()()){
	// Load the raw data that should go into the texture from a file
	std::string sourceFile="BlueNoiseRGBA"+std::to_string(textureResolution)+"x"+std::to_string(textureResolution)+"_"+std::to_string(textureCount)+".blob";
	std::string filePath;
	if(!findFileInDataDirectories(sourceFile,filePath)){
		msgBox("Blue noise with file name "+sourceFile+" could not be located.");
		return;
	}
	std::ifstream file(filePath,std::ios_base::binary);
	std::size_t fileSize=textureCount*textureResolution*textureResolution*4*sizeof(uint16_t);
	std::vector<char> vData(fileSize);
	file.read(vData.data(),fileSize);
	mpTextureArray=Texture::create2D(textureResolution,textureResolution,ResourceFormat::RGBA16Unorm,textureCount,1,vData.data());
}


void BlueNoiseSampler::setProgramVariables(GraphicsVars* pProgramVariables,bool useFixedNumbers){
	pProgramVariables->setTexture("blueNoiseTextureArray",mpTextureArray);
	glm::uvec4 randomVector;
	for(uint i=0;i!=4;++i){
		randomVector[i]=useFixedNumbers?(i*0x123456):randomGenerator();
	}
	pProgramVariables->getConstantBuffer("BlueNoiseCB")->setVariable("gBlueNoiseRandomNumbers",randomVector);
}
