// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


__import Shading;
__import DefaultVS;

cbuffer PerFrameCB:register(b0){
	/*! The width and height of the render target in pixels.*/
	float2 gViewportSize;
};

/*! Performs standard transformations and computes motion vectors.*/
VertexOut mainVS(VertexIn vIn){
	VertexOut result;
	result=defaultVS(vIn);
	result.prevPosH.xy+=result.prevPosH.w*2.0f*float2(gCamera.jitterX,gCamera.jitterY);
	return result;
}

/*! Writes a fragment to the G-buffer.*/
void mainPS(out float4 outDiffuseColor:SV_Target0,out float4 outWorldSpaceNormal:SV_Target1,out float2 outMotionVector:SV_Target2,VertexOut vertexData,float4 pixelCoordinate:SV_POSITION){
	// Prepare shading
	ShadingData shadingData=prepareShadingData(vertexData,gMaterial,gCamera.posW);
	outDiffuseColor=float4(shadingData.diffuse,0.0f);
	outWorldSpaceNormal=float4(mad(shadingData.N,0.5f,0.5f),1.0f);
	outMotionVector=calcMotionVector(pixelCoordinate.xy,vertexData.prevPosH,gViewportSize);
}
